function [NUt,NUm]=LBMpoiseuillevt
%
% Simple LBM code for solving Poiseuille Flow
% D2Q9, periodic boundary conditions, initialization by equilibrium
%
global F rho ux uy

clc
close all
clear all

% Try to reduce it ! up to 1/200
k=1;

% Physics
rho0 = 1.0;
tau = 1/2+3*(1/3*k);
gx = 0.001*k;
gy = 0;

% Numerics
nx = 1;
ny = 30;
nt = 2000/k; % time steps

% lattice D2Q9
V=([0     0
     1     0
     0     1
    -1     0
     0    -1
     1     1
    -1     1
    -1    -1
     1    -1]);
Q=size(V,1);
% bounce back
BB=[1;4;5;2;3;8;9;6;7];

% Initialization (by equilibrium)
for i=1:nx;
    for j=1:ny;
        rho(i,j)=rho0;
        ux(i,j)=0;
        uy(i,j)=0;        
        Feq=Equilibrium(rho(i,j),ux(i,j),uy(i,j));
        F(i,j,:)=Feq(:);
    end
end

for t=1:nt;
%    if (mod(t,100)==0)
%        t
%    end
    % Collision step
    for i=1:nx;
        for j=1:ny;
            [rho(i,j),ux(i,j),uy(i,j)]=Moments(F(i,j,:),V);
            Feq=Equilibrium(rho(i,j),ux(i,j),uy(i,j));
            G=Force(rho(i,j),gx,gy);
            Floc=reshape(F(i,j,:),Q,1);
            Fp(i,j,:)=Floc+1/tau.*(Feq-Floc)+G;
        end
    end
    
    % Streaming step
    for i=1:nx;
        for j=1:ny;
            for alpha=1:Q;
                if ( (j+V(alpha,2))>ny )
                    % Upper wall
                    F(i,j,BB(alpha)) = Fp(i,j,alpha);
                elseif ( (j+V(alpha,2))<1 )
                    % Lower wall
                    F(i,j,BB(alpha)) = Fp(i,j,alpha);
                elseif ( (i+V(alpha,1))>nx )
                    % Right periodic
                    F(1,j+V(alpha,2),alpha) = Fp(i,j,alpha);
                elseif ( (i+V(alpha,1))<1 )
                    % Left periodic
                    F(nx,j+V(alpha,2),alpha) = Fp(i,j,alpha);
                else
                    F(i+V(alpha,1),j+V(alpha,2),alpha) = Fp(i,j,alpha);
                end
            end
        end
    end
    
    % start again
end

NUt = 1/3*(tau-1/2);
NUm = gx*ny^2/(8*max(max(ux)));

%y=[1:ny]-0.5;
%plot(y,ux,'o-'),grid on,hold on
%xmin = 0;
%xmax = ny+0.5;
%ymin = 0.0;
%ymax = max(max(ux))+0.05;
%axis([xmin xmax ymin ymax])
%xlabel('y axis')
%ylabel('u_x velocity')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Feq]=Equilibrium(rho,ux,uy)

Feq=[-2/9*rho*(-2+3*ux^2+3*uy^2);...
    1/18*rho*(6*ux+6*ux^2+2-3*uy^2);...
    -1/18*rho*(-6*uy-6*uy^2-2+3*ux^2);...
    1/18*rho*(-6*ux+6*ux^2+2-3*uy^2);...
    -1/18*rho*(6*uy-6*uy^2-2+3*ux^2);...
    1/36*rho*(9*ux*uy+3*ux+3*uy+1+3*ux^2+3*uy^2);...
    1/36*rho*(-9*ux*uy-3*ux+3*uy+1+3*ux^2+3*uy^2);...
    1/36*rho*(9*ux*uy-3*ux-3*uy+1+3*ux^2+3*uy^2);...
    1/36*rho*(-9*ux*uy+3*ux-3*uy+1+3*ux^2+3*uy^2)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [G]=Force(rho,gx,gy)

G=[0;...
  1/2*rho*gx;...
  1/2*rho*gy;...
 -1/2*rho*gx;...
 -1/2*rho*gy;0;0;0;0];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [rho,ux,uy]=Moments(F,V)

rho = sum(F);
ux = sum(V(:,1).*F(:))/rho;
uy = sum(V(:,2).*F(:))/rho;
